/**
 ******************************************************************************
 * @project  WebClient Weather information viewer example
 * @author   Edward Ahn
 * @version  V1.1
 * @date     07-JAN-2016
 * @brief    Ethernet Communication
 * @MCU      ARM Cortex-M0+
*******************************************************************************
**/

/*
* mac_address : 6 바이트 배열의 MAC 주소
* local_ip : 로컬 IP 주소
* dns_server : DNS 서버 주소
* gateway : 게이트웨이 주소. 디폴트값은 IP와 동일하고 마지막 바이트만 1인 값
* subnet : 서브넷 마스크. 디폴트값은 255.255.255.0
* 반환값 : DHCP를 사용하는 경우 IP 주소 할당에 성공하면 1을 반환하고 실패하면 0을
  반환한다. DHCP를 사용하지 않는 경우 반환값은 없다.
*/

/* 날씨 정보 요청
client.connect(server, 80), 80번 포트를 통해 server - api.openweathermap.org에 연결을 시도하게 된다.
연결에 성공하게 된다면 Client는 "GET /data/2.5/weather?q=seoul,kr&appid=2de143494c0b295cca9337e1e96b00e0&mode=xml" 
명령어를 서버로 전송하게 되며 데이터를 요청하게 된다. 요청시 appid=2de143494c0b295cca9337e1e96b00e0 위 Key값을 같이 보내야 값이 정상적으로 온다. 
*/

/*
  string.substring(from, to)
  string: a variable of type String
  from: the index to start the substring at
  to (optional): the index to end the substring before
*/

/* getInt 함수 
XML Data를 Parsing하여 Print하면 String값으로 ex)"275.89"가 된다.
그런데, String값으로만 보기엔 온도 데이터가 이상하다.
우리가 원하는 온도 값을 보기 위해선 온도데이터를 정수형으로 변환 후 계산을 이행해야한다.
위 처럼 "" 표시를 제거한 뒤 inter형으로 변환하고 '-273'을 한다.
*/

#include <SPI.h>                      // W5500은 SPI 통신을 사용한다.
#include <Ethernet2.h>                 // 이더넷 라이브러리

String Stringbuffer = "";             // 서버에서 전송된 데이터 String저장
String temp = "";                     // 온도 저장 변수
String hum = "";                      // 습도 저장 변수
String times = "";                     // 시간 정보 변수

boolean readTemp = false;             //온도 데이터가 있는지 여부 판단 Flag
boolean readHum = false;              //습도 데이터가 있는지 여부 판단 Flag
boolean readTime = false;             //시간 데이터가 있는지 여부 판단 Flag

/* MAC address Setting */
#if defined(WIZ550io_WITH_MACADDRESS) // WIZ550io를 사용하면 MAC주소가 따로 필요 없음
;
#else
byte mac[] = {0x00, 0x08, 0xDC, 0x50, 0xf4, 0xc2};
#endif  

#define __USE_DHCP__ // DHCP사용 시 Define

char server[] = "api.openweathermap.org"; // 접속할 Server 도메인

/* 로컬 네트워크 값 세팅 */
IPAddress ip(192,168,0, 177); // TCP/IP 네트워크에 있는 호스트를 식별하는 숫자
IPAddress gateway(192,168,0, 1); // 다른 네트워크에 있는 호스트와 통신할 때 사용
IPAddress subnet(255, 255, 255, 0); // 호스트가 로컬 or 원격 네트워크에 있는지 확인할 때 사용
IPAddress myDns(8, 8, 8, 8); // google puble dns

EthernetClient client; // 클라이언트 선언


int getInt(String input){  // String데이터를 intger형으로 변환하는 함수
  int i = 2;
  int temp;

  while(input[i] != '"'){ // 포인터로 " 건너뛰기
    i++;
  }
  input = input.substring(2,i); // 건너뛴 사이의 값을 읽어 input에 저장
  char carray[20]; // 값을 저장할 버퍼(배열) 생성
  //Serial.println(input);
  /*  
    input의 String값을 carray(버퍼)로 복사한다.
    위 작업이 선행되어야 값이 정상적으로 삽입된다.
    이유는, 이전까지는 포인터로 값을 가르키기만 한건데, 위 작업으로 값이 변수에 저장된 것이다.
  */
  input.toCharArray(carray, sizeof(carray)); 
  //Serial.println(carray);
  temp = atoi(carray); // 정수형으로 변환
  //Serial.println(temp);
  return temp;
}

// Weather information Request Function
void httpRequest(void) {
  if (client.connect(server, 80)) { // 서버로 접속 시도..
    SerialUSB.println("connected"); // 연결 성공
    // 날씨 정보 요청
    client.println("GET /data/2.5/weather?q=seoul,kr&id=524901&APPID=52236a3cfa5c0912faa03cf18f9874ff&mode=xml HTTP/1.0\n\n");
    client.println("HOST: api.openweathermap.org");
    client.println("Connection: close");
    client.println();
    SerialUSB.println("-----------------------------------------------------------");
    SerialUSB.println("It is Seoul, republic of korea, Temperature & humidity Data");
    SerialUSB.println("-----------------------------------------------------------");
  }
  else { // 서버와 연결 실패 시 실행한다
    SerialUSB.println("connection failed");
  }
}
  
void setup() {
  //각 변수에 정해진 공간 할당
  Stringbuffer.reserve(100);
  temp.reserve(10);
  hum.reserve(10);
  times.reserve(20);
  
  // 이더넷 디바이스 initialize
  #if defined __USE_DHCP__ // DHCP 사용 시 (네트워크 정보 DHCP에서 할당받음)
  #if defined(WIZ550io_WITH_MACADDRESS) // WIZ550io에 할당된 MAC 주소를 사용할 경우
    Ethernet.begin();
  #else
    Ethernet.begin(mac); // MAC이 없는 이더넷 디바이스의 경우
  #endif
  #else // Static 주소로 사용 시 (네트워크 정보 사용자가 임의로 지정)
  #if defined(WIZ550io_WITH_MACADDRESS) 
    Ethernet.begin(ip, myDns, gateway, subnet);
  #else
    Ethernet.begin(mac, ip, myDns, gateway, subnet);
  #endif
  #endif
  // give the Ethernet shield a second to initialize:
  delay(1000);
  SerialUSB.begin(9600); // 시리얼 통신 초기화
  // WizArduino M0(Zero)는 컴파일할 때 BOOT모드로 변경되었다가 App으로 동작하게 된다. 
  // 이로인해 시리얼을 사용할 때 Delay 6초를 해준다. (업로드 시 MCU가 BOOT로 갔다가 App으로 넘어오는 시간이 6초이기 때문) 
  delay(6000);
  
  /* Client Network information 출력 */
  /* DHCP로 구동 시 IP 정보가 나온다면 정상적으로 DHCP로 부터 IP 할당 받음 */
  SerialUSB.print("Client IP : ");
  SerialUSB.println(Ethernet.localIP());
  SerialUSB.print("Client subnetMask : ");
  SerialUSB.println(Ethernet.subnetMask());
  SerialUSB.print("Client gatewayIP : ");
  SerialUSB.println(Ethernet.gatewayIP());
  SerialUSB.println("connecting...");
  
  httpRequest(); // 날씨 정보 요청
}

void loop()
{
  while (client.available()) { // 서버로부터 수신된 데이터가 있는 경우
    char c = client.read(); // Client는 서버에서 전송한 데이터가 존재한다면 Client.read()를 통해 c 에서 한글자씩 저장한다.
    Stringbuffer += c; // c 에서 저장된 문자는 currentLine에 한 글자씩 쌓이게되어 String(문자열)이 된다.
        
    /* 온도 값 읽기*/
    if (Stringbuffer.endsWith("<temperature value=")) {
      // 현재 문자열이 "<temperature value="로 끝났다면 온도데이터를 받을 준비를 한다.
      readTemp = true; // Flag
      temp = "";
    }
    if (readTemp) {
      if (c != 'm') { // 전송될 문자가 'm'이 올때까지 온도값으로 인식
        temp += c;
      } 
      else { // 전송된 문자가 'm'이라면 온도데이터를 그만 저장하고 온도값 출력
        readTemp = false;
        SerialUSB.print("-  Temperature: ");
        SerialUSB.print(getInt(temp)-273);
        SerialUSB.println((char)176);    //degree symbol
        }
     }
     
     /* 습도 값 읽기 */
     if ( Stringbuffer.endsWith("<humidity value=")) {
        // 현재 문자열이 "<humidity value ="로 끝났다면 습도 데이터를 받을 준비를 한다.
       readHum = true; 
       hum = "";
     }
     if (readHum) { 
       if (c != 'u') {// 전송될 문자가 'u'가 아니라면 계속 습도값을 받게 된다.
         hum += c;
       } 
       else { // 다음에 전송된 문자가 'u'라면 습도값을 그만 저장하고 습도값 출력
         readHum = false;
         SerialUSB.print("-  Humidity: ");
         SerialUSB.print(getInt(hum));
         SerialUSB.println((char)37);
       }
     }
     
     // 시간 값 읽기
     if ( Stringbuffer.endsWith("<lastupdate value=")) {
        // 현재 문자열이 "<lastupdata value="로 끝났다면 마지막 업데이트 시간 데이터를 받을 준비를 한다.
        readTime = true; 
        times = "";
      }


      if (readTime) {
        if (c != '/') { // 다음 전송될 문자가 '/'가 아니라면 계속 시간데이터를 받는다
          times += c;
        } 
        else { // 다음에 전송된 문자가 '/'라면 시간데이터 그만 저장하고 데이터 출력
          readTime = false;
          SerialUSB.print("-  Last update: ");
          SerialUSB.println(times.substring(2,times.length()-1));
        }
      }
    
    if (!client.connected()) { // 연결 종료
      SerialUSB.println();
      SerialUSB.println("disconnecting.");
      client.stop();
      // do nothing forevermore:
      while(true);
    }
  }
}
